@@ -1,4 +1,4 @@
-/* $OpenBSD: e_des3.c,v 1.20 2019/05/14 15:40:44 beck Exp $ */
+/* $OpenBSD: e_des3.c,v 1.28 2023/07/07 19:37:53 beck Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -56,6 +56,7 @@
  * [including the GNU Public Licence.]
  */
 
+#include <limits.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -67,15 +68,7 @@
 #include <openssl/evp.h>
 #include <openssl/objects.h>
 
-#include "evp_locl.h"
-
-static int des_ede_init_key(EVP_CIPHER_CTX *ctx, const unsigned char *key,
-    const unsigned char *iv, int enc);
-
-static int des_ede3_init_key(EVP_CIPHER_CTX *ctx, const unsigned char *key,
-    const unsigned char *iv, int enc);
-
-static int des3_ctrl(EVP_CIPHER_CTX *c, int type, int arg, void *ptr);
+#include "evp_local.h"
 
 typedef struct {
     DES_key_schedule ks1;/* key schedule */
@@ -85,147 +78,6 @@ typedef struct {
 
 #define data(ctx) ((DES_EDE_KEY *)(ctx)->cipher_data)
 
-/* Because of various casts and different args can't use IMPLEMENT_BLOCK_CIPHER */
-
-static int
-des_ede_ecb_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
-    const unsigned char *in, size_t inl)
-{
-	BLOCK_CIPHER_ecb_loop()
-	DES_ecb3_encrypt((const_DES_cblock *)(in + i), (DES_cblock *)(out + i),
-	    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3, ctx->encrypt);
-	return 1;
-}
-
-static int
-des_ede_ofb_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
-    const unsigned char *in, size_t inl)
-{
-	while (inl >= EVP_MAXCHUNK) {
-		DES_ede3_ofb64_encrypt(in, out, (long)EVP_MAXCHUNK,
-		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
-		    (DES_cblock *)ctx->iv, &ctx->num);
-		inl -= EVP_MAXCHUNK;
-		in += EVP_MAXCHUNK;
-		out += EVP_MAXCHUNK;
-	}
-	if (inl)
-		DES_ede3_ofb64_encrypt(in, out, (long)inl,
-		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
-		    (DES_cblock *)ctx->iv, &ctx->num);
-
-	return 1;
-}
-
-static int
-des_ede_cbc_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
-    const unsigned char *in, size_t inl)
-{
-	while (inl >= EVP_MAXCHUNK) {
-		DES_ede3_cbc_encrypt(in, out, (long)EVP_MAXCHUNK,
-		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
-		    (DES_cblock *)ctx->iv, ctx->encrypt);
-		inl -= EVP_MAXCHUNK;
-		in += EVP_MAXCHUNK;
-		out += EVP_MAXCHUNK;
-	}
-	if (inl)
-		DES_ede3_cbc_encrypt(in, out, (long)inl,
-		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
-		    (DES_cblock *)ctx->iv, ctx->encrypt);
-	return 1;
-}
-
-static int
-des_ede_cfb64_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
-    const unsigned char *in, size_t inl)
-{
-	while (inl >= EVP_MAXCHUNK) {
-		DES_ede3_cfb64_encrypt(in, out, (long)EVP_MAXCHUNK,
-		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
-		    (DES_cblock *)ctx->iv, &ctx->num, ctx->encrypt);
-		inl -= EVP_MAXCHUNK;
-		in += EVP_MAXCHUNK;
-		out += EVP_MAXCHUNK;
-	}
-	if (inl)
-		DES_ede3_cfb64_encrypt(in, out, (long)inl,
-		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
-		    (DES_cblock *)ctx->iv, &ctx->num, ctx->encrypt);
-	return 1;
-}
-
-/* Although we have a CFB-r implementation for 3-DES, it doesn't pack the right
-   way, so wrap it here */
-static int
-des_ede3_cfb1_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
-    const unsigned char *in, size_t inl)
-{
-	size_t n;
-	unsigned char c[1], d[1];
-	if (!(ctx->flags & EVP_CIPH_FLAG_LENGTH_BITS))
-		inl *= 8;
-
-	for (n = 0; n < inl; ++n) {
-		c[0] = (in[n/8]&(1 << (7 - n % 8))) ? 0x80 : 0;
-		DES_ede3_cfb_encrypt(c, d, 1, 1,
-		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
-		    (DES_cblock *)ctx->iv, ctx->encrypt);
-		out[n / 8] = (out[n / 8] & ~(0x80 >> (unsigned int)(n % 8))) |
-		    ((d[0] & 0x80) >> (unsigned int)(n % 8));
-	}
-
-	return 1;
-}
-
-static int
-des_ede3_cfb8_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
-    const unsigned char *in, size_t inl)
-{
-	while (inl >= EVP_MAXCHUNK) {
-		DES_ede3_cfb_encrypt(in, out, 8, (long)EVP_MAXCHUNK,
-		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
-		    (DES_cblock *)ctx->iv, ctx->encrypt);
-		inl -= EVP_MAXCHUNK;
-		in += EVP_MAXCHUNK;
-		out += EVP_MAXCHUNK;
-	}
-	if (inl)
-		DES_ede3_cfb_encrypt(in, out, 8, (long)inl,
-		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
-		    (DES_cblock *)ctx->iv, ctx->encrypt);
-	return 1;
-}
-
-BLOCK_CIPHER_defs(des_ede, DES_EDE_KEY, NID_des_ede, 8, 16, 8, 64,
-    EVP_CIPH_RAND_KEY, des_ede_init_key, NULL,
-    EVP_CIPHER_set_asn1_iv,
-    EVP_CIPHER_get_asn1_iv,
-    des3_ctrl)
-
-#define des_ede3_cfb64_cipher des_ede_cfb64_cipher
-#define des_ede3_ofb_cipher des_ede_ofb_cipher
-#define des_ede3_cbc_cipher des_ede_cbc_cipher
-#define des_ede3_ecb_cipher des_ede_ecb_cipher
-
-BLOCK_CIPHER_defs(des_ede3, DES_EDE_KEY, NID_des_ede3, 8, 24, 8, 64,
-    EVP_CIPH_RAND_KEY, des_ede3_init_key, NULL,
-    EVP_CIPHER_set_asn1_iv,
-    EVP_CIPHER_get_asn1_iv,
-    des3_ctrl)
-
-BLOCK_CIPHER_def_cfb(des_ede3, DES_EDE_KEY, NID_des_ede3, 24, 8, 1,
-    EVP_CIPH_RAND_KEY, des_ede3_init_key, NULL,
-    EVP_CIPHER_set_asn1_iv,
-    EVP_CIPHER_get_asn1_iv,
-    des3_ctrl)
-
-BLOCK_CIPHER_def_cfb(des_ede3, DES_EDE_KEY, NID_des_ede3, 24, 8, 8,
-    EVP_CIPH_RAND_KEY, des_ede3_init_key, NULL,
-    EVP_CIPHER_set_asn1_iv,
-    EVP_CIPHER_get_asn1_iv,
-    des3_ctrl)
-
 static int
 des_ede_init_key(EVP_CIPHER_CTX *ctx, const unsigned char *key,
     const unsigned char *iv, int enc)
@@ -272,6 +124,363 @@ des3_ctrl(EVP_CIPHER_CTX *c, int type, int arg, void *ptr)
 	}
 }
 
+static int
+des_ede_ecb_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
+    const unsigned char *in, size_t inl)
+{
+	size_t i, bl;
+
+	bl = ctx->cipher->block_size;
+
+	if (inl < bl)
+		return 1;
+
+	inl -= bl;
+
+	for (i = 0; i <= inl; i += bl)
+		DES_ecb3_encrypt((const_DES_cblock *)(in + i), (DES_cblock *)(out + i),
+		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3, ctx->encrypt);
+
+	return 1;
+}
+
+static int
+des_ede_ofb_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
+    const unsigned char *in, size_t inl)
+{
+	size_t chunk = LONG_MAX & ~0xff;
+
+	while (inl >= chunk) {
+		DES_ede3_ofb64_encrypt(in, out, (long)chunk,
+		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
+		    (DES_cblock *)ctx->iv, &ctx->num);
+		inl -= chunk;
+		in += chunk;
+		out += chunk;
+	}
+	if (inl)
+		DES_ede3_ofb64_encrypt(in, out, (long)inl,
+		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
+		    (DES_cblock *)ctx->iv, &ctx->num);
+
+	return 1;
+}
+
+static int
+des_ede_cbc_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
+    const unsigned char *in, size_t inl)
+{
+	size_t chunk = LONG_MAX & ~0xff;
+
+	while (inl >= chunk) {
+		DES_ede3_cbc_encrypt(in, out, (long)chunk,
+		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
+		    (DES_cblock *)ctx->iv, ctx->encrypt);
+		inl -= chunk;
+		in += chunk;
+		out += chunk;
+	}
+	if (inl)
+		DES_ede3_cbc_encrypt(in, out, (long)inl,
+		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
+		    (DES_cblock *)ctx->iv, ctx->encrypt);
+	return 1;
+}
+
+static int
+des_ede_cfb64_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
+    const unsigned char *in, size_t inl)
+{
+	size_t chunk = LONG_MAX & ~0xff;
+
+	while (inl >= chunk) {
+		DES_ede3_cfb64_encrypt(in, out, (long)chunk,
+		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
+		    (DES_cblock *)ctx->iv, &ctx->num, ctx->encrypt);
+		inl -= chunk;
+		in += chunk;
+		out += chunk;
+	}
+	if (inl)
+		DES_ede3_cfb64_encrypt(in, out, (long)inl,
+		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
+		    (DES_cblock *)ctx->iv, &ctx->num, ctx->encrypt);
+	return 1;
+}
+
+/* Although we have a CFB-r implementation for 3-DES, it doesn't pack the right
+   way, so wrap it here */
+static int
+des_ede3_cfb1_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
+    const unsigned char *in, size_t inl)
+{
+	unsigned char c[1], d[1];
+	size_t n;
+
+	if (!(ctx->flags & EVP_CIPH_FLAG_LENGTH_BITS))
+		inl *= 8;
+
+	for (n = 0; n < inl; ++n) {
+		c[0] = (in[n/8]&(1 << (7 - n % 8))) ? 0x80 : 0;
+		DES_ede3_cfb_encrypt(c, d, 1, 1,
+		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
+		    (DES_cblock *)ctx->iv, ctx->encrypt);
+		out[n / 8] = (out[n / 8] & ~(0x80 >> (unsigned int)(n % 8))) |
+		    ((d[0] & 0x80) >> (unsigned int)(n % 8));
+	}
+
+	return 1;
+}
+
+static int
+des_ede3_cfb8_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
+    const unsigned char *in, size_t inl)
+{
+	size_t chunk = LONG_MAX & ~0xff;
+
+	while (inl >= chunk) {
+		DES_ede3_cfb_encrypt(in, out, 8, (long)chunk,
+		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
+		    (DES_cblock *)ctx->iv, ctx->encrypt);
+		inl -= chunk;
+		in += chunk;
+		out += chunk;
+	}
+	if (inl)
+		DES_ede3_cfb_encrypt(in, out, 8, (long)inl,
+		    &data(ctx)->ks1, &data(ctx)->ks2, &data(ctx)->ks3,
+		    (DES_cblock *)ctx->iv, ctx->encrypt);
+	return 1;
+}
+
+static const EVP_CIPHER des_ede_cbc = {
+	.nid = NID_des_ede_cbc,
+	.block_size = 8,
+	.key_len = 16,
+	.iv_len = 8,
+	.flags = EVP_CIPH_RAND_KEY | EVP_CIPH_CBC_MODE,
+	.init = des_ede_init_key,
+	.do_cipher = des_ede_cbc_cipher,
+	.cleanup = NULL,
+	.ctx_size = sizeof(DES_EDE_KEY),
+	.set_asn1_parameters = EVP_CIPHER_set_asn1_iv,
+	.get_asn1_parameters = EVP_CIPHER_get_asn1_iv,
+	.ctrl = des3_ctrl,
+	.app_data = NULL,
+};
+
+const EVP_CIPHER *
+EVP_des_ede_cbc(void)
+{
+	return &des_ede_cbc;
+}
+
+static const EVP_CIPHER des_ede_cfb64 = {
+	.nid = NID_des_ede_cfb64,
+	.block_size = 1,
+	.key_len = 16,
+	.iv_len = 8,
+	.flags = EVP_CIPH_RAND_KEY | EVP_CIPH_CFB_MODE,
+	.init = des_ede_init_key,
+	.do_cipher = des_ede_cfb64_cipher,
+	.cleanup = NULL,
+	.ctx_size = sizeof(DES_EDE_KEY),
+	.set_asn1_parameters = EVP_CIPHER_set_asn1_iv,
+	.get_asn1_parameters = EVP_CIPHER_get_asn1_iv,
+	.ctrl = des3_ctrl,
+	.app_data = NULL,
+};
+
+const EVP_CIPHER *
+EVP_des_ede_cfb64(void)
+{
+	return &des_ede_cfb64;
+}
+
+static const EVP_CIPHER des_ede_ofb = {
+	.nid = NID_des_ede_ofb64,
+	.block_size = 1,
+	.key_len = 16,
+	.iv_len = 8,
+	.flags = EVP_CIPH_RAND_KEY | EVP_CIPH_OFB_MODE,
+	.init = des_ede_init_key,
+	.do_cipher = des_ede_ofb_cipher,
+	.cleanup = NULL,
+	.ctx_size = sizeof(DES_EDE_KEY),
+	.set_asn1_parameters = EVP_CIPHER_set_asn1_iv,
+	.get_asn1_parameters = EVP_CIPHER_get_asn1_iv,
+	.ctrl = des3_ctrl,
+	.app_data = NULL,
+};
+
+const EVP_CIPHER *
+EVP_des_ede_ofb(void)
+{
+	return &des_ede_ofb;
+}
+
+static const EVP_CIPHER des_ede_ecb = {
+	.nid = NID_des_ede_ecb,
+	.block_size = 8,
+	.key_len = 16,
+	.iv_len = 0,
+	.flags = EVP_CIPH_RAND_KEY | EVP_CIPH_ECB_MODE,
+	.init = des_ede_init_key,
+	.do_cipher = des_ede_ecb_cipher,
+	.cleanup = NULL,
+	.ctx_size = sizeof(DES_EDE_KEY),
+	.set_asn1_parameters = EVP_CIPHER_set_asn1_iv,
+	.get_asn1_parameters = EVP_CIPHER_get_asn1_iv,
+	.ctrl = des3_ctrl,
+	.app_data = NULL,
+};
+
+const EVP_CIPHER *
+EVP_des_ede_ecb(void)
+{
+	return &des_ede_ecb;
+}
+
+
+#define des_ede3_cfb64_cipher des_ede_cfb64_cipher
+#define des_ede3_ofb_cipher des_ede_ofb_cipher
+#define des_ede3_cbc_cipher des_ede_cbc_cipher
+#define des_ede3_ecb_cipher des_ede_ecb_cipher
+
+static const EVP_CIPHER des_ede3_cbc = {
+	.nid = NID_des_ede3_cbc,
+	.block_size = 8,
+	.key_len = 24,
+	.iv_len = 8,
+	.flags = EVP_CIPH_RAND_KEY | EVP_CIPH_CBC_MODE,
+	.init = des_ede3_init_key,
+	.do_cipher = des_ede3_cbc_cipher,
+	.cleanup = NULL,
+	.ctx_size = sizeof(DES_EDE_KEY),
+	.set_asn1_parameters = EVP_CIPHER_set_asn1_iv,
+	.get_asn1_parameters = EVP_CIPHER_get_asn1_iv,
+	.ctrl = des3_ctrl,
+	.app_data = NULL,
+};
+
+const EVP_CIPHER *
+EVP_des_ede3_cbc(void)
+{
+	return &des_ede3_cbc;
+}
+
+static const EVP_CIPHER des_ede3_cfb64 = {
+	.nid = NID_des_ede3_cfb64,
+	.block_size = 1,
+	.key_len = 24,
+	.iv_len = 8,
+	.flags = EVP_CIPH_RAND_KEY | EVP_CIPH_CFB_MODE,
+	.init = des_ede3_init_key,
+	.do_cipher = des_ede3_cfb64_cipher,
+	.cleanup = NULL,
+	.ctx_size = sizeof(DES_EDE_KEY),
+	.set_asn1_parameters = EVP_CIPHER_set_asn1_iv,
+	.get_asn1_parameters = EVP_CIPHER_get_asn1_iv,
+	.ctrl = des3_ctrl,
+	.app_data = NULL,
+};
+
+const EVP_CIPHER *
+EVP_des_ede3_cfb64(void)
+{
+	return &des_ede3_cfb64;
+}
+
+static const EVP_CIPHER des_ede3_ofb = {
+	.nid = NID_des_ede3_ofb64,
+	.block_size = 1,
+	.key_len = 24,
+	.iv_len = 8,
+	.flags = EVP_CIPH_RAND_KEY | EVP_CIPH_OFB_MODE,
+	.init = des_ede3_init_key,
+	.do_cipher = des_ede3_ofb_cipher,
+	.cleanup = NULL,
+	.ctx_size = sizeof(DES_EDE_KEY),
+	.set_asn1_parameters = EVP_CIPHER_set_asn1_iv,
+	.get_asn1_parameters = EVP_CIPHER_get_asn1_iv,
+	.ctrl = des3_ctrl,
+	.app_data = NULL,
+};
+
+const EVP_CIPHER *
+EVP_des_ede3_ofb(void)
+{
+	return &des_ede3_ofb;
+}
+
+static const EVP_CIPHER des_ede3_ecb = {
+	.nid = NID_des_ede3_ecb,
+	.block_size = 8,
+	.key_len = 24,
+	.iv_len = 0,
+	.flags = EVP_CIPH_RAND_KEY | EVP_CIPH_ECB_MODE,
+	.init = des_ede3_init_key,
+	.do_cipher = des_ede3_ecb_cipher,
+	.cleanup = NULL,
+	.ctx_size = sizeof(DES_EDE_KEY),
+	.set_asn1_parameters = EVP_CIPHER_set_asn1_iv,
+	.get_asn1_parameters = EVP_CIPHER_get_asn1_iv,
+	.ctrl = des3_ctrl,
+	.app_data = NULL,
+};
+
+const EVP_CIPHER *
+EVP_des_ede3_ecb(void)
+{
+	return &des_ede3_ecb;
+}
+
+
+static const EVP_CIPHER des_ede3_cfb1 = {
+	.nid = NID_des_ede3_cfb1,
+	.block_size = 1,
+	.key_len = 24,
+	.iv_len = 8,
+	.flags = EVP_CIPH_RAND_KEY | EVP_CIPH_CFB_MODE,
+	.init = des_ede3_init_key,
+	.do_cipher = des_ede3_cfb1_cipher,
+	.cleanup = NULL,
+	.ctx_size = sizeof(DES_EDE_KEY),
+	.set_asn1_parameters = EVP_CIPHER_set_asn1_iv,
+	.get_asn1_parameters = EVP_CIPHER_get_asn1_iv,
+	.ctrl = des3_ctrl,
+	.app_data = NULL,
+};
+
+const EVP_CIPHER *
+EVP_des_ede3_cfb1(void)
+{
+	return &des_ede3_cfb1;
+}
+
+
+static const EVP_CIPHER des_ede3_cfb8 = {
+	.nid = NID_des_ede3_cfb8,
+	.block_size = 1,
+	.key_len = 24,
+	.iv_len = 8,
+	.flags = EVP_CIPH_RAND_KEY | EVP_CIPH_CFB_MODE,
+	.init = des_ede3_init_key,
+	.do_cipher = des_ede3_cfb8_cipher,
+	.cleanup = NULL,
+	.ctx_size = sizeof(DES_EDE_KEY),
+	.set_asn1_parameters = EVP_CIPHER_set_asn1_iv,
+	.get_asn1_parameters = EVP_CIPHER_get_asn1_iv,
+	.ctrl = des3_ctrl,
+	.app_data = NULL,
+};
+
+const EVP_CIPHER *
+EVP_des_ede3_cfb8(void)
+{
+	return &des_ede3_cfb8;
+}
+
 const EVP_CIPHER *
 EVP_des_ede(void)
 {
