@@ -15,14 +15,11 @@
  * limitations under the License.
  */
 
-#if defined(_MSC_VER)
-	#pragma warning ( disable: 4231 4251 4275 4786 )
-#endif
-
 #include <log4cxx/logstring.h>
 #include <log4cxx/pattern/propertiespatternconverter.h>
 #include <log4cxx/spi/loggingevent.h>
 #include <log4cxx/spi/location/locationinfo.h>
+#include <log4cxx/private/patternconverter_priv.h>
 
 #include <iterator>
 
@@ -31,12 +28,26 @@ using namespace log4cxx::pattern;
 using namespace log4cxx::spi;
 using namespace log4cxx::helpers;
 
+#define priv static_cast<PropertiesPatternConverterPrivate*>(m_priv.get())
+
+struct PropertiesPatternConverter::PropertiesPatternConverterPrivate : public PatternConverterPrivate
+{
+	PropertiesPatternConverterPrivate( const LogString& name, const LogString& style, const LogString& propertyName ) :
+		PatternConverterPrivate( name, style ),
+		option(propertyName) {}
+
+	/**
+	 * Name of property to output.
+	 */
+	const LogString option;
+};
+
 IMPLEMENT_LOG4CXX_OBJECT(PropertiesPatternConverter)
 
 PropertiesPatternConverter::PropertiesPatternConverter(const LogString& name1,
 	const LogString& propertyName) :
-	LoggingEventPatternConverter(name1, LOG4CXX_STR("property")),
-	option(propertyName)
+	LoggingEventPatternConverter(
+		std::make_unique<PropertiesPatternConverterPrivate>(name1, LOG4CXX_STR("property"), propertyName))
 {
 }
 
@@ -45,17 +56,15 @@ PatternConverterPtr PropertiesPatternConverter::newInstance(
 {
 	if (options.size() == 0)
 	{
-		static PatternConverterPtr def(new PropertiesPatternConverter(
-				LOG4CXX_STR("Properties"), LOG4CXX_STR("")));
+		static PatternConverterPtr def = std::make_shared<PropertiesPatternConverter>(
+				LOG4CXX_STR("Properties"), LOG4CXX_STR(""));
 		return def;
 	}
 
 	LogString converterName(LOG4CXX_STR("Property{"));
 	converterName.append(options[0]);
 	converterName.append(LOG4CXX_STR("}"));
-	PatternConverterPtr converter(new PropertiesPatternConverter(
-			converterName, options[0]));
-	return converter;
+	return std::make_shared<PropertiesPatternConverter>(converterName, options[0]);
 }
 
 void PropertiesPatternConverter::format(
@@ -63,7 +72,7 @@ void PropertiesPatternConverter::format(
 	LogString& toAppendTo,
 	Pool& /* p */) const
 {
-	if (option.length() == 0)
+	if (priv->option.length() == 0)
 	{
 		toAppendTo.append(1, (logchar) 0x7B /* '{' */);
 
@@ -85,7 +94,7 @@ void PropertiesPatternConverter::format(
 	}
 	else
 	{
-		event->getMDC(option, toAppendTo);
+		event->getMDC(priv->option, toAppendTo);
 	}
 }
 
