@@ -14,18 +14,26 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2008 Koji Otani <sho@bbr.jp>
-// Copyright (C) 2012, 2017 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2012, 2017, 2021, 2023 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2012 Hib Eris <hib@hiberis.nl>
-// Copyright (C) 2016, 2018-2020 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2016, 2018-2022 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2016 Jason Crain <jason@aquaticape.us>
 // Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 // Copyright (C) 2018, 2020 Nelson Benítez León <nbenitezl@gmail.com>
+// Copyright (C) 2021 Georgiy Sgibnev <georgiy@sgibnev.com>. Work sponsored by lab50.net.
+// Copyright (C) 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2023 Even Rouault <even.rouault@spatialys.com>
+// Copyright (C) 2023 Oliver Sander <oliver.sander@tu-dresden.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
 //
 //========================================================================
 
+#if defined(MIKTEX_WINDOWS)
+#define MIKTEX_UTF8_WRAP_ALL 1
+#include <miktex/utf8wrap.h>
+#endif
 #include "goo/gmem.h"
 #include "PDFDocEncoding.h"
 #include "GlobalParams.h"
@@ -34,6 +42,8 @@
 #include "UnicodeMapFuncs.h"
 #include <algorithm>
 
+#include <config.h>
+
 bool UnicodeIsValid(Unicode ucs4)
 {
     return (ucs4 < 0x110000) && ((ucs4 & 0xfffff800) != 0xd800) && (ucs4 < 0xfdd0 || ucs4 > 0xfdef) && ((ucs4 & 0xfffe) != 0xfffe);
@@ -52,8 +62,9 @@ int UTF16toUCS4(const Unicode *utf16, int utf16Len, Unicode **ucs4_out)
         }
         len++;
     }
-    if (ucs4_out == nullptr)
+    if (ucs4_out == nullptr) {
         return len;
+    }
 
     u = (Unicode *)gmallocn(len, sizeof(Unicode));
     n = 0;
@@ -85,24 +96,24 @@ int UTF16toUCS4(const Unicode *utf16, int utf16Len, Unicode **ucs4_out)
     return len;
 }
 
-int TextStringToUCS4(const GooString *textStr, Unicode **ucs4)
+int TextStringToUCS4(const std::string &textStr, Unicode **ucs4)
 {
     int i, len;
     const char *s;
     Unicode *u;
     bool isUnicode, isUnicodeLE;
 
-    len = textStr->getLength();
-    s = textStr->c_str();
+    len = textStr.size();
+    s = textStr.c_str();
     if (len == 0) {
         *ucs4 = nullptr;
         return 0;
     }
 
-    if (textStr->hasUnicodeMarker()) {
+    if (GooString::hasUnicodeMarker(textStr)) {
         isUnicode = true;
         isUnicodeLE = false;
-    } else if (textStr->hasUnicodeMarkerLE()) {
+    } else if (GooString::hasUnicodeMarkerLE(textStr)) {
         isUnicode = false;
         isUnicodeLE = true;
     } else {
@@ -116,10 +127,11 @@ int TextStringToUCS4(const GooString *textStr, Unicode **ucs4)
         if (len > 0) {
             utf16 = new Unicode[len];
             for (i = 0; i < len; i++) {
-                if (isUnicode)
+                if (isUnicode) {
                     utf16[i] = (s[2 + i * 2] & 0xff) << 8 | (s[3 + i * 2] & 0xff);
-                else // UnicodeLE
+                } else { // UnicodeLE
                     utf16[i] = (s[3 + i * 2] & 0xff) << 8 | (s[2 + i * 2] & 0xff);
+                }
             }
             len = UTF16toUCS4(utf16, len, &u);
             delete[] utf16;
@@ -176,377 +188,28 @@ static const uint32_t UTF8_REJECT = 12;
 static const uint32_t UCS4_MAX = 0x10FFFF;
 static const Unicode REPLACEMENT_CHAR = 0xFFFD;
 
+// clang-format off
 static const uint8_t decodeUtf8Table[] = {
-    // The first part of the table maps bytes to character classes
-    // to reduce the size of the transition table and create bitmasks.
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0, // 00..1f
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0, // 20..3f
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0, // 40..5f
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0, // 60..7f
-    1,
-    1,
-    1,
-    1,
-    1,
-    1,
-    1,
-    1,
-    1,
-    1,
-    1,
-    1,
-    1,
-    1,
-    1,
-    1,
-    9,
-    9,
-    9,
-    9,
-    9,
-    9,
-    9,
-    9,
-    9,
-    9,
-    9,
-    9,
-    9,
-    9,
-    9,
-    9, // 80..9f
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7,
-    7, // a0..bf
-    8,
-    8,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2,
-    2, // c0..df
-    10,
-    3,
-    3,
-    3,
-    3,
-    3,
-    3,
-    3,
-    3,
-    3,
-    3,
-    3,
-    3,
-    4,
-    3,
-    3,
-    11,
-    6,
-    6,
-    6,
-    5,
-    8,
-    8,
-    8,
-    8,
-    8,
-    8,
-    8,
-    8,
-    8,
-    8,
-    8, // e0..ff
+  // The first part of the table maps bytes to character classes
+  // to reduce the size of the transition table and create bitmasks.
+   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, // 00..1f
+   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, // 20..3f
+   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, // 40..5f
+   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, // 60..7f
+   1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9, // 80..9f
+   7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,  7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7, // a0..bf
+   8,8,2,2,2,2,2,2,2,2,2,2,2,2,2,2,  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2, // c0..df
+  10,3,3,3,3,3,3,3,3,3,3,3,3,4,3,3, 11,6,6,6,5,8,8,8,8,8,8,8,8,8,8,8, // e0..ff
 
-    // The second part is a transition table that maps a combination
-    // of a state of the automaton and a character class to a state.
-    0,
-    12,
-    24,
-    36,
-    60,
-    96,
-    84,
-    12,
-    12,
-    12,
-    48,
-    72,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    0,
-    12,
-    12,
-    12,
-    12,
-    12,
-    0,
-    12,
-    0,
-    12,
-    12,
-    12,
-    24,
-    12,
-    12,
-    12,
-    12,
-    12,
-    24,
-    12,
-    24,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    24,
-    12,
-    12,
-    12,
-    12,
-    12,
-    24,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    24,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    36,
-    12,
-    36,
-    12,
-    12,
-    12,
-    36,
-    12,
-    12,
-    12,
-    12,
-    12,
-    36,
-    12,
-    36,
-    12,
-    12,
-    12,
-    36,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
-    12,
+  // The second part is a transition table that maps a combination
+  // of a state of the automaton and a character class to a state.
+   0,12,24,36,60,96,84,12,12,12,48,72, 12,12,12,12,12,12,12,12,12,12,12,12,
+  12, 0,12,12,12,12,12, 0,12, 0,12,12, 12,24,12,12,12,12,12,24,12,24,12,12,
+  12,12,12,12,12,12,12,24,12,12,12,12, 12,24,12,12,12,12,12,12,12,24,12,12,
+  12,12,12,12,12,12,12,36,12,36,12,12, 12,36,12,12,12,12,12,36,12,36,12,12,
+  12,36,12,12,12,12,12,12,12,12,12,12,
 };
+// clang-format on
 
 // Decode utf8 state machine for fast UTF-8 decoding. Initialise state
 // to 0 and call decodeUtf8() for each byte of UTF-8. Return value
@@ -566,6 +229,55 @@ inline uint32_t decodeUtf8(uint32_t *state, uint32_t *codep, char byte)
     return *state;
 }
 
+int utf8CountUCS4(const char *utf8)
+{
+    uint32_t codepoint;
+    uint32_t state = 0;
+    int count = 0;
+
+    while (*utf8) {
+        decodeUtf8(&state, &codepoint, *utf8);
+        if (state == UTF8_ACCEPT) {
+            count++;
+        } else if (state == UTF8_REJECT) {
+            count++; // replace with REPLACEMENT_CHAR
+            state = 0;
+        }
+        utf8++;
+    }
+    if (state != UTF8_ACCEPT && state != UTF8_REJECT) {
+        count++; // replace with REPLACEMENT_CHAR
+    }
+
+    return count;
+}
+
+int utf8ToUCS4(const char *utf8, Unicode **ucs4_out)
+{
+    int len = utf8CountUCS4(utf8);
+    Unicode *u = (Unicode *)gmallocn(len, sizeof(Unicode));
+    int n = 0;
+    uint32_t codepoint;
+    uint32_t state = 0;
+
+    while (*utf8 && n < len) {
+        decodeUtf8(&state, &codepoint, *utf8);
+        if (state == UTF8_ACCEPT) {
+            u[n++] = codepoint;
+        } else if (state == UTF8_REJECT) {
+            u[n++] = REPLACEMENT_CHAR; // invalid byte for this position
+            state = 0;
+        }
+        utf8++;
+    }
+    if (state != UTF8_ACCEPT && state != UTF8_REJECT) {
+        u[n] = REPLACEMENT_CHAR; // invalid byte for this position
+    }
+
+    *ucs4_out = u;
+    return len;
+}
+
 // Count number of UTF-16 code units required to convert a UTF-8 string
 // (excluding terminating NULL). Each invalid byte is counted as a
 // code point since the UTF-8 conversion functions will replace it with
@@ -579,20 +291,22 @@ int utf8CountUtf16CodeUnits(const char *utf8)
     while (*utf8) {
         decodeUtf8(&state, &codepoint, *utf8);
         if (state == UTF8_ACCEPT) {
-            if (codepoint < 0x10000)
+            if (codepoint < 0x10000) {
                 count++;
-            else if (codepoint <= UCS4_MAX)
+            } else if (codepoint <= UCS4_MAX) {
                 count += 2;
-            else
+            } else {
                 count++; // replace with REPLACEMENT_CHAR
+            }
         } else if (state == UTF8_REJECT) {
             count++; // replace with REPLACEMENT_CHAR
             state = 0;
         }
         utf8++;
     }
-    if (state != UTF8_ACCEPT && state != UTF8_REJECT)
+    if (state != UTF8_ACCEPT && state != UTF8_REJECT) {
         count++; // replace with REPLACEMENT_CHAR
+    }
 
     return count;
 }
@@ -639,8 +353,9 @@ int utf8ToUtf16(const char *utf8, uint16_t *utf16, int maxUtf16, int maxUtf8)
         *p++ = REPLACEMENT_CHAR;
         nOut++;
     }
-    if (nOut > maxUtf16 - 1)
+    if (nOut > maxUtf16 - 1) {
         nOut = maxUtf16 - 1;
+    }
     utf16[nOut] = 0;
     return nOut;
 }
@@ -648,14 +363,37 @@ int utf8ToUtf16(const char *utf8, uint16_t *utf16, int maxUtf16, int maxUtf8)
 // Allocate utf16 string and convert utf8 into it.
 uint16_t *utf8ToUtf16(const char *utf8, int *len)
 {
+    if (isUtf8WithBom(utf8)) {
+        utf8 += 3;
+    }
     int n = utf8CountUtf16CodeUnits(utf8);
-    if (len)
+    if (len) {
         *len = n;
+    }
     uint16_t *utf16 = (uint16_t *)gmallocn(n + 1, sizeof(uint16_t));
-    utf8ToUtf16(utf8, utf16);
+    utf8ToUtf16(utf8, utf16, n + 1, INT_MAX);
     return utf16;
 }
 
+std::string utf8ToUtf16WithBom(const std::string &utf8)
+{
+    if (utf8.empty()) {
+        return {};
+    }
+    int tmp_length; // Number of UTF-16 symbols.
+    char *tmp_str = (char *)utf8ToUtf16(utf8.c_str(), &tmp_length);
+#ifndef WORDS_BIGENDIAN
+    for (int i = 0; i < tmp_length; i++) {
+        std::swap(tmp_str[i * 2], tmp_str[i * 2 + 1]);
+    }
+#endif
+
+    std::string result(unicodeByteOrderMark);
+    result.append(tmp_str, tmp_length * 2);
+    gfree(tmp_str);
+    return result;
+}
+
 static const uint32_t UTF16_ACCEPT = 0;
 static const uint32_t UTF16_REJECT = -1;
 
@@ -698,24 +436,26 @@ int utf16CountUtf8Bytes(const uint16_t *utf16)
     while (*utf16) {
         decodeUtf16(&state, &codepoint, *utf16);
         if (state == UTF16_ACCEPT) {
-            if (codepoint < 0x80)
+            if (codepoint < 0x80) {
                 count++;
-            else if (codepoint < 0x800)
+            } else if (codepoint < 0x800) {
                 count += 2;
-            else if (codepoint < 0x10000)
+            } else if (codepoint < 0x10000) {
                 count += 3;
-            else if (codepoint <= UCS4_MAX)
+            } else if (codepoint <= UCS4_MAX) {
                 count += 4;
-            else
+            } else {
                 count += 3; // replace with REPLACEMENT_CHAR
+            }
         } else if (state == UTF16_REJECT) {
             count += 3; // replace with REPLACEMENT_CHAR
             state = 0;
         }
         utf16++;
     }
-    if (state != UTF8_ACCEPT && state != UTF8_REJECT)
+    if (state != UTF8_ACCEPT && state != UTF8_REJECT) {
         count++; // replace with REPLACEMENT_CHAR
+    }
 
     return count;
 }
@@ -759,8 +499,9 @@ int utf16ToUtf8(const uint16_t *utf16, char *utf8, int maxUtf8, int maxUtf16)
         nOut += count;
         nOut++;
     }
-    if (nOut > maxUtf8 - 1)
+    if (nOut > maxUtf8 - 1) {
         nOut = maxUtf8 - 1;
+    }
     utf8[nOut] = 0;
     return nOut;
 }
@@ -769,8 +510,9 @@ int utf16ToUtf8(const uint16_t *utf16, char *utf8, int maxUtf8, int maxUtf16)
 char *utf16ToUtf8(const uint16_t *utf16, int *len)
 {
     int n = utf16CountUtf8Bytes(utf16);
-    if (len)
+    if (len) {
         *len = n;
+    }
     char *utf8 = (char *)gmalloc(n + 1);
     utf16ToUtf8(utf16, utf8);
     return utf8;
@@ -788,13 +530,14 @@ void unicodeToAscii7(const Unicode *in, int len, Unicode **ucs4_out, int *out_le
     }
 
     if (indices) {
-        if (!in_idx)
+        if (!in_idx) {
             indices = nullptr;
-        else
+        } else {
             idx = (int *)gmallocn(len * 8 + 1, sizeof(int));
+        }
     }
 
-    GooString gstr;
+    std::string str;
 
     char buf[8]; // 8 is enough for mapping an unicode char to a string
     int i, n, k;
@@ -807,17 +550,50 @@ void unicodeToAscii7(const Unicode *in, int len, Unicode **ucs4_out, int *out_le
             buf[0] = 31;
             n = 1;
         }
-        gstr.append(buf, n);
+        str.append(buf, n);
         if (indices) {
-            for (; n > 0; n--)
+            for (; n > 0; n--) {
                 idx[k++] = in_idx[i];
+            }
         }
     }
 
-    *out_len = TextStringToUCS4(&gstr, ucs4_out);
+    *out_len = TextStringToUCS4(str, ucs4_out);
 
     if (indices) {
         idx[k] = in_idx[len];
         *indices = idx;
     }
 }
+
+// Convert a PDF Text String to UTF-8
+//   textStr    - PDF text string
+//   returns UTF-8 string.
+std::string TextStringToUtf8(const std::string &textStr)
+{
+    int i, len;
+    const char *s;
+    char *utf8;
+
+    len = textStr.size();
+    s = textStr.c_str();
+    if (GooString::hasUnicodeMarker(textStr)) {
+        uint16_t *utf16;
+        len = len / 2 - 1;
+        utf16 = new uint16_t[len];
+        for (i = 0; i < len; i++) {
+            utf16[i] = (s[2 + i * 2] & 0xff) << 8 | (s[3 + i * 2] & 0xff);
+        }
+        utf8 = utf16ToUtf8(utf16, &len);
+        delete[] utf16;
+    } else {
+        utf8 = (char *)gmalloc(len + 1);
+        for (i = 0; i < len; i++) {
+            utf8[i] = pdfDocEncoding[s[i] & 0xff];
+        }
+        utf8[i] = 0;
+    }
+    std::string utf8_string(utf8);
+    gfree(utf8);
+    return utf8_string;
+}
