@@ -9,6 +9,9 @@
  * Copyright (C) 2018-2020 Adam Reichold <adam.reichold@t-online.de>
  * Copyright (C) 2019, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
  * Copyright (C) 2019 João Netto <joaonetto901@gmail.com>
+ * Copyright (C) 2021 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>
+ * Copyright (C) 2021 Mahmoud Khalil <mahmoudkhalil11@gmail.com>
+ * Copyright (C) 2023 Shivodit Gill <shivodit.gill@gmail.com>
  * Inspired on code by
  * Copyright (C) 2004 by Albert Astals Cid <tsdgeos@terra.es>
  * Copyright (C) 2004 by Enrico Ros <eros.kde@email.it>
@@ -28,6 +31,10 @@
  * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#if defined(MIKTEX_WINDOWS)
+#define MIKTEX_UTF8_WRAP_ALL 1
+#include <miktex/utf8wrap.h>
+#endif
 #include "poppler-private.h"
 #include "poppler-form.h"
 
@@ -40,6 +47,15 @@
 #include <PDFDocEncoding.h>
 #include <UnicodeMap.h>
 
+#ifdef ANDROID
+#    include <QtCore/QString>
+#    include <QtCore/QDir>
+#    include <QtCore/QFile>
+#    include <QtCore/QFileInfo>
+#    include <QtCore/QStandardPaths>
+#    include <QtCore/QDirIterator>
+#endif
+
 namespace Poppler {
 
 namespace Debug {
@@ -99,8 +115,9 @@ QString UnicodeParsedString(const GooString *s1)
 
 QString UnicodeParsedString(const std::string &s1)
 {
-    if (s1.empty())
+    if (s1.empty()) {
         return QString();
+    }
 
     if (GooString::hasUnicodeMarker(s1) || GooString::hasUnicodeMarkerLE(s1)) {
         return QString::fromUtf16(reinterpret_cast<const ushort *>(s1.c_str()), s1.size() / 2);
@@ -115,6 +132,9 @@ QString UnicodeParsedString(const std::string &s1)
 
 GooString *QStringToUnicodeGooString(const QString &s)
 {
+    if (s.isEmpty()) {
+        return new GooString();
+    }
     int len = s.length() * 2 + 2;
     char *cstring = (char *)gmallocn(len, sizeof(char));
     cstring[0] = (char)0xfe;
@@ -132,8 +152,9 @@ GooString *QStringToGooString(const QString &s)
 {
     int len = s.length();
     char *cstring = (char *)gmallocn(s.length(), sizeof(char));
-    for (int i = 0; i < len; ++i)
+    for (int i = 0; i < len; ++i) {
         cstring[i] = s.at(i).unicode();
+    }
     GooString *ret = new GooString(cstring, len);
     gfree(cstring);
     return ret;
@@ -178,8 +199,9 @@ Annot::AdditionalActionsType toPopplerAdditionalActionType(Annotation::Additiona
 
 static void linkActionToTocItem(const ::LinkAction *a, DocumentData *doc, QDomElement *e)
 {
-    if (!a || !e)
+    if (!a || !e) {
         return;
+    }
 
     switch (a->getKind()) {
     case actionGoTo: {
@@ -192,8 +214,9 @@ static void linkActionToTocItem(const ::LinkAction *a, DocumentData *doc, QDomEl
             // so better storing the reference and provide the viewport on demand
             const GooString *s = g->getNamedDest();
             QChar *charArray = new QChar[s->getLength()];
-            for (int i = 0; i < s->getLength(); ++i)
+            for (int i = 0; i < s->getLength(); ++i) {
                 charArray[i] = QChar(s->c_str()[i]);
+            }
             QString aux(charArray, s->getLength());
             e->setAttribute(QStringLiteral("DestinationName"), aux);
             delete[] charArray;
@@ -213,8 +236,9 @@ static void linkActionToTocItem(const ::LinkAction *a, DocumentData *doc, QDomEl
             // so better storing the reference and provide the viewport on demand
             const GooString *s = g->getNamedDest();
             QChar *charArray = new QChar[s->getLength()];
-            for (int i = 0; i < s->getLength(); ++i)
+            for (int i = 0; i < s->getLength(); ++i) {
                 charArray[i] = QChar(s->c_str()[i]);
+            }
             QString aux(charArray, s->getLength());
             e->setAttribute(QStringLiteral("DestinationName"), aux);
             delete[] charArray;
@@ -246,6 +270,31 @@ void DocumentData::init()
     paperColor = Qt::white;
     m_hints = 0;
     m_optContentModel = nullptr;
+    xrefReconstructed = false;
+    xrefReconstructedCallback = {};
+
+#ifdef ANDROID
+    // Copy fonts from android apk to the app's storage dir, and
+    // set the font directory path
+    QString assetsFontDir = QStringLiteral("assets:/share/fonts");
+    QString fontsdir = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + QStringLiteral("/fonts");
+    QDir fontPath = QDir(fontsdir);
+
+    if (fontPath.mkpath(fontPath.absolutePath())) {
+        GlobalParams::setFontDir(fontPath.absolutePath().toStdString());
+        QDirIterator iterator(assetsFontDir, QDir::NoFilter, QDirIterator::Subdirectories);
+
+        while (iterator.hasNext()) {
+            iterator.next();
+            QFileInfo fontFileInfo = iterator.fileInfo();
+            QString fontFilePath = assetsFontDir + QStringLiteral("/") + fontFileInfo.fileName();
+            QString destPath = fontPath.absolutePath() + QStringLiteral("/") + fontFileInfo.fileName();
+            QFile::copy(fontFilePath, destPath);
+        }
+    } else {
+        GlobalParams::setFontDir("");
+    }
+#endif
 }
 
 void DocumentData::addTocChildren(QDomDocument *docSyn, QDomNode *parent, const std::vector<::OutlineItem *> *items)
@@ -258,8 +307,9 @@ void DocumentData::addTocChildren(QDomDocument *docSyn, QDomNode *parent, const
         const Unicode *uniChar = outlineItem->getTitle();
         int titleLength = outlineItem->getTitleLength();
         name = unicodeToQString(uniChar, titleLength);
-        if (name.isEmpty())
+        if (name.isEmpty()) {
             continue;
+        }
 
         QDomElement item = docSyn->createElement(name);
         parent->appendChild(item);
@@ -273,8 +323,20 @@ void DocumentData::addTocChildren(QDomDocument *docSyn, QDomNode *parent, const
         // 3. recursively descend over children
         outlineItem->open();
         const std::vector<::OutlineItem *> *children = outlineItem->getKids();
-        if (children)
+        if (children) {
             addTocChildren(docSyn, &item, children);
+        }
+    }
+}
+
+void DocumentData::noitfyXRefReconstructed()
+{
+    if (!xrefReconstructed) {
+        xrefReconstructed = true;
+    }
+
+    if (xrefReconstructedCallback) {
+        xrefReconstructedCallback();
     }
 }
 
@@ -287,5 +349,4 @@ FormFieldIconData *FormFieldIconData::getData(const FormFieldIcon &f)
 {
     return f.d_ptr;
 }
-
 }
