@@ -1,6 +1,6 @@
-/* 1-3.cpp:
+/* 3-1.cpp:
 
-   Copyright (C) 2018 Christian Schenk
+   Copyright (C) 1996-2017 Christian Schenk
 
    This file is part of the MiKTeX Core Library.
 
@@ -23,16 +23,18 @@
 
 #include <miktex/Core/Test>
 
-#include <string>
-
 using namespace MiKTeX::Test;
-using namespace std;
 
-BEGIN_TEST_SCRIPT("process-1-3");
+BEGIN_TEST_SCRIPT("process-3-1");
 
 BEGIN_TEST_FUNCTION(1);
 {
-  throw TestException("process-1-3", "{a}errorMessage{b}", "{a}description{b}", "{a}remedy{b}", "tag", { "a", "x", "b", "y" }, MIKTEX_SOURCE_LOCATION());
+  char buf[100];
+  size_t n;
+  while ((n = fread(buf, 1, 100, stdin)) > 0)
+  {
+    fwrite(buf, 1, n, stdout);
+  }
 }
 END_TEST_FUNCTION();
 
