@@ -2818,11 +2818,16 @@ BEGIN
           case UBLOCK_CJK_UNIFIED_IDEOGRAPHS_EXTENSION_D:
           case UBLOCK_CJK_UNIFIED_IDEOGRAPHS_EXTENSION_E:
           case UBLOCK_CJK_UNIFIED_IDEOGRAPHS_EXTENSION_F:
-#if defined(U_ICU_VERSION_MAJOR_NUM) && (U_ICU_VERSION_MAJOR_NUM > 65)
+#if defined(U_ICU_VERSION_MAJOR_NUM)
+#if U_ICU_VERSION_MAJOR_NUM > 65
           case UBLOCK_CJK_UNIFIED_IDEOGRAPHS_EXTENSION_G:
 #endif
-#if defined(U_ICU_VERSION_MAJOR_NUM) && (U_ICU_VERSION_MAJOR_NUM > 71)
+#if U_ICU_VERSION_MAJOR_NUM > 71
           case UBLOCK_CJK_UNIFIED_IDEOGRAPHS_EXTENSION_H:
+#endif
+#if U_ICU_VERSION_MAJOR_NUM > 73
+          case UBLOCK_CJK_UNIFIED_IDEOGRAPHS_EXTENSION_I:
+#endif
 #endif
             string_width |= 0x001;
             break;
@@ -2831,11 +2836,13 @@ BEGIN
           case UBLOCK_KATAKANA:
           case UBLOCK_KATAKANA_PHONETIC_EXTENSIONS:
           case UBLOCK_KANA_EXTENDED_A:
-#if defined(U_ICU_VERSION_MAJOR_NUM) && (U_ICU_VERSION_MAJOR_NUM > 69)
+#if defined(U_ICU_VERSION_MAJOR_NUM)
+#if U_ICU_VERSION_MAJOR_NUM > 69
           case UBLOCK_KANA_EXTENDED_B:
 #endif
-#if defined(U_ICU_VERSION_MAJOR_NUM) && (U_ICU_VERSION_MAJOR_NUM > 63)
+#if U_ICU_VERSION_MAJOR_NUM > 63
           case UBLOCK_SMALL_KANA_EXTENSION:
+#endif
 #endif
             string_width |= 0x002;
             break;
