@@ -2,7 +2,7 @@
 ** BoundingBox.cpp                                                      **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2023 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,11 +18,15 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#if defined(MIKTEX)
+#include <config.h>
+#endif
 #include <algorithm>
 #include <sstream>
 #include "BoundingBox.hpp"
 #include "Matrix.hpp"
 #include "utility.hpp"
+#include "SVGTree.hpp"
 #include "XMLNode.hpp"
 #include "XMLString.hpp"
 
@@ -76,7 +80,7 @@ vector<Length> BoundingBox::extractLengths (string boxstr) {
 	vector<string> lengthStrings = util::split(boxstr, " ");
 	for (const string &lenstr : lengthStrings) {
 		if (!lenstr.empty())
-			lengths.emplace_back(Length(lenstr));
+			lengths.emplace_back(lenstr);
 	}
 	return lengths;
 }
@@ -246,7 +250,7 @@ void BoundingBox::transform (const Matrix &tm) {
 }
 
 
-string BoundingBox::toSVGViewBox () const {
+string BoundingBox::svgViewBoxString () const {
 	ostringstream oss;
 	oss << XMLString(_ulx) << ' ' << XMLString(_uly) << ' ' << XMLString(width()) << ' ' << XMLString(height());
 	return oss.str();
@@ -272,3 +276,18 @@ unique_ptr<XMLElement> BoundingBox::createSVGRect () const {
 	rect->addAttribute("fill", "none");
 	return rect;
 }
+
+
+unique_ptr<XMLElement> BoundingBox::createSVGPath () const {
+	GraphicsPath<double> path;
+	path.moveto(minX(), minY());
+	path.lineto(maxX(), minY());
+	path.lineto(maxX(), maxY());
+	path.lineto(minX(), maxY());
+	path.closepath();
+	ostringstream oss;
+	path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
+	auto pathElem = util::make_unique<XMLElement>("path");
+	pathElem->addAttribute("d", oss.str());
+	return pathElem;
+}
\ No newline at end of file
