@@ -1,6 +1,6 @@
 /* gsf2pk.cpp: Gsf-to-Pk converter (based on gsftopk)
 
-   Copyright (C) 2004-2020 Christian Schenk
+   Copyright (C) 2004-2021 Christian Schenk
    Copyright (C) 1993-2000 Paul Vojta
 
    Permission is hereby granted, free of charge, to any person
@@ -49,6 +49,10 @@
 #include <miktex/Util/StringUtil>
 #include <miktex/Wrappers/PoptWrapper>
 
+#if defined(MIKTEX_WINDOWS)
+#include <miktex/Core/win/ConsoleCodePageSwitcher>
+#endif
+
 #include "gsf2pk-version.h"
 
 using namespace MiKTeX::App;
@@ -1488,6 +1492,9 @@ void Converter::Main(int argc, const char** argv)
 
 int MAIN(int argc, MAINCHAR** argv)
 {
+#if defined(MIKTEX_WINDOWS)
+  ConsoleCodePageSwitcher cpSwitcher;
+#endif
   Converter conv;
   try
   {
@@ -1513,14 +1520,14 @@ int MAIN(int argc, MAINCHAR** argv)
 
   catch (const MiKTeXException& e)
   {
-    Application::Sorry("gsf2pk", e);
+    conv.Sorry("gsf2pk", e);
     e.Save();
     return 1;
   }
 
   catch (const exception& e)
   {
-    Application::Sorry("gsf2pk", e);
+    conv.Sorry("gsf2pk", e);
     return 1;
   }
 
