@@ -7,7 +7,7 @@
 --     copyright = 'LuaTeX Development Team',
 -- }
 
-LUATEXCOREVERSION = 1.120 -- we reflect the luatex version where changes happened
+LUATEXCOREVERSION = 1.161 -- we reflect the luatex version where changes happened
 
 -- This file overloads some Lua functions. The readline variants provide the same
 -- functionality as LuaTeX <= 1.04 and doing it this way permits us to keep the
@@ -16,10 +16,13 @@ LUATEXCOREVERSION = 1.120 -- we reflect the luatex version where changes happene
 -- We test for functions already being defined so that we don't overload ones that
 -- are provided in the startup script.
 
+
+
 local saferoption = status.safer_option
 local shellescape = status.shell_escape -- 0 (disabled) 1 (anything) 2 (restricted)
 local kpseused    = status.kpse_used    -- 0 1
 
+
 if kpseused == 1 then
 
     local type = type
@@ -33,8 +36,12 @@ if kpseused == 1 then
     local kpse_recordinputfile  = kpse.record_input_file
     local kpse_recordoutputfile = kpse.record_output_file
 
+    local kpse_out_name_ok = kpse.out_name_ok
+    local kpse_in_name_ok  = kpse.in_name_ok
+
+
     local io_open               = io.open
-    local io_popen              = io.popen
+    -- local io_popen              = io.popen -- not need, we  use os.kpsepopen
     local io_lines              = io.lines
 
     local fio_readline          = fio.readline
@@ -48,7 +55,13 @@ if kpseused == 1 then
         if not how then
             how = 'r'
         end
-        local f = io_open(name,how)
+        local check = true 
+        if how == 'r' or how == 'rb' or how == '' then 
+           check = kpse.in_name_ok(name)
+        else 
+           check = kpse.out_name_ok(name)
+        end  
+        local f = check and io_open(name,how)
         if f then
             if type(how) == 'string' and find(how,'w') then
                 kpse_recordoutputfile(name,'w')
@@ -68,19 +81,24 @@ if kpseused == 1 then
                 how = 'r'
             end
         end
-        local f = io_open(name,how)
+        local check = false 
+        if how == 'r' or how == 'rb' or how == '' then 
+           check = kpse.in_name_ok(name)
+        end  
+        local f = check and io_open(name,how)
         if f then
             fio_recordfilename(name,'r')
         end
         return f
     end
 
-    local function luatex_io_popen(name,...)
-        local okay, found = kpse_checkpermission(name)
-        if okay and found then
-            return io_popen(found,...)
-        end
-    end
+    --  not need, we  use os.kpsepopen 
+    --local function luatex_io_popen(name,...)
+    --    local okay, found = kpse_checkpermission(name)
+    --    if okay and found then
+    --        return io_popen(found,...)
+    --    end
+    --end
 
     -- local function luatex_io_lines(name,how)
     --     if name then
@@ -102,7 +120,8 @@ if kpseused == 1 then
 
     local function luatex_io_lines(name,how)
         if type(name) == "string" then
-            local f = io_open(name,how or 'r')
+            local check = kpse.in_name_ok(name)
+            local f = check and io_open(name,how or 'r')
             if f then
                 return function()
                     local l = fio_readline(f)
@@ -130,7 +149,8 @@ if kpseused == 1 then
     mt.lines = luatex_io_readline
 
     io.open  = luatex_io_open
-    io.popen = luatex_io_popen
+    --io.popen = luatex_io_popen -- not need, we  use os.kpsepopen
+    io.popen = os.kpsepopen
 
 else
 
@@ -169,6 +189,8 @@ if saferoption == 1 then
     os.setenv  = installdummy("os.setenv")
     os.tempdir = installdummy("os.tempdir")
 
+    os.kpsepopen = installdummy("os.kpsepopen")
+
     io.popen   = installdummy("io.popen")
     io.open    = installdummy("io.open",luatex_io_open_readonly)
 
@@ -198,6 +220,9 @@ if saferoption == 1 or shellescape ~= 1 then
     package.searchers[4] = nil
     package.searchers[3] = nil
 
+    if os.setenv then
+        os.setenv = function(...) end
+    end
     ffi = require('ffi')
 
     if ffi then
@@ -209,6 +234,8 @@ if saferoption == 1 or shellescape ~= 1 then
     end
 
     ffi = nil
+    package.loaded.ffi = nil
+    package.preload.ffi = error
 
 end
 
